/*
 * Scanner for a simple interface description language.
 *
 * Copyright (C) 2019 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

%{
#include <string.h>
#include "types.h"
#include "y.tab.h"
%}

BEGINCOMMENT                  "/*"
ENDCOMMENT                    "*/"
WS                            [ \t\n]+

%x comment

%%

{BEGINCOMMENT}                BEGIN(comment);

<comment>{ENDCOMMENT}         BEGIN(INITIAL);
<comment>[^*]+                /* discard */
<comment>"*"[^/]              /* discard */

{WS}                          /* discard */

^"#include"                   return INCLUDE;
"<"[^>]*">"                   yylval.str = strdup(yytext); return HEADER;
\"[^"]*\"                     yylval.str = strdup(yytext); return HEADER;

"interface"                   return INTERFACE;

"in"                          yylval.num = IN_PARAMETER; return IN;
"inout"                       yylval.num = INOUT_PARAMETER; return INOUT;
"out"                         yylval.num = OUT_PARAMETER; return OUT;

"cap"                         yylval.num = CAP_ITEM; return CAP;
"fpage"                       yylval.num = FPAGE_ITEM; return FPAGE;

"("                           return LPAR;
")"                           return RPAR;

"["                           return LBRACE;
"]"                           return RBRACE;

","                           return SEP;
";"                           return TERM;

"{"                           return BEGINSECTION;
"}"                           return ENDSECTION;

[_[:alpha:]][_[:alnum:]]*     yylval.str = strdup(yytext); return IDENTIFIER;
[[:digit:]]+                  yylval.str = strdup(yytext); return DECIMAL;
"0"[xX][[:xdigit:]]+          yylval.str = strdup(yytext); return HEXADECIMAL;

%%
